import os, pdb, time, warnings
import ctypes as ct
import numpy as np

from cudamat import cuda_sync_threads
from cudamat import deprecated
from cudamat import dot
from cudamat import empty
from cudamat import sum
from cudamat import CUDAMatException
from cudamat import get_last_cuda_error
from cudamat import generate_exception

_cmextras = ct.cdll.LoadLibrary('libcudamatextras.so')

_cmextras.get_last_cuda_error.restype = ct.c_char_p

_cmextras.tiled_to_flat_diag.restype = ct.c_int
_cmextras.flat_to_tiled_diag.restype = ct.c_int

def dot_block(m1, block_size1, t1, m2, block_size2, t2, target):
    """
    m1 is a matrix with shape (h1, num_blocks * block_size1)
    m2 is a matrix with shape (h2, num_blocks * block_size2)

    dot_block multiplies each h1 x block_size1 block of m1 by the corresponding
    h2 x block_size2 block of m2.

    flags t1 and t2 denote whether each block should be transposed before multiplying.
    """

    num_blocks = m1.shape[1]/block_size1

    if t2:
        t_block_size = m2.shape[0]
    else:
        t_block_size = block_size2

    for i in range(num_blocks):
        m1_slice = m1.get_col_slice(i * block_size1, (i + 1) * block_size1)
        m2_slice = m2.get_col_slice(i * block_size2, (i + 1) * block_size2)

        t_slice = target.get_col_slice(i * t_block_size, (i + 1) * t_block_size)

        if t1:
            m1_slice = m1_slice.T

        if t2:
            m2_slice = m2_slice.T

        dot(m1_slice, m2_slice, target = t_slice)

    cuda_sync_threads()

def flat_to_tiled_diag(source, target, num_channels, num_tiles, offset, num_offsets):
    img_width = ct.c_uint(int(np.sqrt(source.shape[0]/num_channels)))
    tile_width = ct.c_uint(int(np.sqrt(target.shape[0]/num_channels)))

    _cmextras.flat_to_tiled_diag(source.p_mat, target.p_mat, 
                                 img_width,
                                 ct.c_uint(num_channels),
                                 tile_width,
                                 ct.c_uint(num_tiles),
                                 ct.c_uint(offset),
                                 ct.c_uint(num_offsets),
                                 ct.c_uint(source.shape[1]))

def tiled_to_flat_diag(source, target, num_channels, num_tiles, offset, num_offsets):
    img_width = ct.c_uint(int(np.sqrt(target.mat.size[0]/num_channels)))
    tile_width = ct.c_uint(int(np.sqrt(source.mat.size[0]/num_channels)))

    _cmextras.tiled_to_flat_diag(source.p_mat, target.p_mat, 
                                 img_width,
                                 ct.c_uint(num_channels),
                                 tile_width,
                                 ct.c_uint(num_tiles),
                                 ct.c_uint(offset),
                                 ct.c_uint(num_offsets),
                                 ct.c_uint(target.shape[1]))

