# visualize the mean and the covariance filters (matrix W and C) of mPoT
#
# Marc'Aurelio Ranzato
# 17 December 2010
# Marc'Aurelio Ranzato

import sys
from pylab import *
import numpy as np
from scipy.io import loadmat, savemat
import pdb
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from scipy import mgrid

# from http://www.scipy.org 
def imsave(filename, X, **kwargs):
    """ Homebrewed imsave to have nice colors... """
    figsize=(array(X.shape)/100.0)[::-1]
    rcParams.update({'figure.figsize':figsize})
    fig = figure(figsize=figsize)
    axes([0,0,1,1]) # Make the plot occupy the whole canvas
    axis('off')
    fig.set_size_inches(figsize)
    imshow(X,origin='upper', **kwargs)
    savefig(filename, facecolor='black', edgecolor='black', dpi=400)
    close(fig)


def plot_filters(mm, filename, show_plot):
    (nrv, nrh) = mm.shape
    npr = np.sqrt(nrv)
    npc = npr
    nr = np.floor(np.sqrt(nrh))
    nc = np.ceil(nrh / nr)
    coef = 127./max([np.abs(mm.min()), np.abs(mm.max())]) 
    ff = np.zeros((npr,npc))
    ofs = 1
    ima = np.zeros((ofs+nr*(ofs+npr),ofs+nc*(ofs+npc)))
    for cc in range(nrh):
        ff = np.reshape(mm[:,cc],(npr,npc),order='F').copy()
        ff = ff*coef + 128.    
        ima[ofs + np.floor(cc/nc)*(ofs+npr) : ofs + (1 + floor(cc/nc))*(ofs+npr) - ofs, ofs + np.mod(cc,nc)*(ofs+npc) : ofs + (1 + mod(cc,nc))*(ofs+npc) - ofs] = ff.copy()
    
    imsave(filename, ima, cmap=cm.gray)
    if show_plot == 1:
        plt.figure()
        imshow(ima, cmap=cm.gray, interpolation='nearest')
        axis('off')
        suptitle(filename,fontsize=12)
        plt.show()
        raw_input("press a button to continue")
        plt.close()
        

if __name__ == "__main__":
    mm = loadmat("temp_params.mat")
    pca = loadmat("PCAtransf_patches_8x8_0meanPerPatch_unitstd.mat")
    show_plot = 0

    # plot covariance filters
    plot_filters(np.dot(pca["invpcatransf"], mm["VF"]), "covariance_filters", show_plot)

    # plot mean filters
    plot_filters(mm["w_mean"], "mean_filters", show_plot)

    # plot negative particles
    plot_filters(mm["rawnegdata"], "negative_particles", show_plot)


